-- INCLUDE.EX --------------------------------------------------------------

-- Program to search through the files in c:\euphoria\include, looking for
-- a procedure with a given name and to then display the first few lines of
-- the procedure, if found.

----------------------------------------------------------------------------

    global constant     ESC         = 27,
			BAD_FILE    = -1

include file.e      -- dir()
include graphics.e  -- for get_position()
include get.e       -- for value()

   constant path = "c:\\euphoria\\include\\"

procedure blankline(atom row)
    -- blanks the screen line specified by row
    position(row, 1)
    puts(1, repeat(' ', 80))
    position (row, 1)
end procedure

function rtrim(object a)
  -- crop trailing spaces off object a
    if equal(length(a), 0) then 
      a = " "
    end if
    while a[length(a)] = ' ' do
      if equal(a, " ") then
	 a = ""
	 exit
      end if
      a = a[1..length(a)-1] 
    end while
  return a
end function
  
function ltrim(object a)
  -- crop leading spaces off object a
    if equal(length(a), 0) then 
      a = " "
    end if
    while match(" ", a) = 1 do
      if equal(a, " ") then
	 a = ""
	 exit
      end if
      a = a[2..length(a)]   
    end while 
  return a
end function
  
function build_index(sequence file_list)
   -- Constructs an index to global procedures/functions in the files specified
   -- in file_list (a list generated by dir()). 
    sequence index, criteria
    object getter
    atom fn
    -- will be looking in files for lines containing these words:
    criteria = {"global function", "global procedure"}
    index = {}                                      -- initially, index is empty
    for i = 1 to length(file_list) do               -- for each item in the list
	index = append(index, {file_list[i][1], {}})--  add filename and seq 
    end for
    -- Each entry in index is now:
    --      {filename}, {empty_sequence}
    for i = 1 to length(index) do                   -- for each index entry
	fn = open(path & index[i][1], "r")          --  open relevant file
	if fn != BAD_FILE then                      --  if opened ok
	    getter = " "                            --    initial to loop
	    while sequence(getter) do               --    until eof
		getter = gets(fn)                   --      read line
		if sequence(getter) then            --      if read ok
		    getter = ltrim(getter)          --        remove lead spcs
		    getter = getter[1..$-1]         --        remove newline
		    for j = 1 to 2 do               --        for each criterion
		      if match(criteria[j], getter) then --     if a match
			getter = getter[8..$]            --      chop 'global '
			index[i][2] = append(index[i][2], getter)-- and add
		      end if
		    end for
		end if
	    end while
	    close(fn)                               -- close the file
	end if
    end for
    return index
end function

procedure search_display(sequence searcher, sequence index)
-- Displays the first 30-odd lines of the header file function/procedure
-- specified by searcher.
    sequence filename
    object getter
    atom fn
    for i = 1 to length(index) do                       -- for each index item
	filename = index[i][1]                          --  This is the filename
	for j = 1 to length(index[i][2]) do             --  for each sub-entry
	    if match(searcher, index[i][2][j]) then     --   if it on hit list
		fn = open(path & filename, "r")         --     open the file
		if fn != BAD_FILE then                  --     if opened ok
		    getter = " "                        --       init to loop
		    while sequence(getter) do           --       while not eof
			getter = gets(fn)               --        read a line
			if sequence(getter) then        --        if valid line
			  if match(index[i][2][j], getter) then -- seek in line
			    text_color(YELLOW)
			    printf
			      (1, "INCLUDE FILE = %s\n", {filename})-- on match
			    text_color(BRIGHT_WHITE)
			    puts(1, getter)             --         write line
			    for k = 1 to 38 do          --         for 38 lines
				getter = gets(fn)       --          get a line
				if sequence(getter) then--          if ok
				    printf(1, " %s", {getter})--      write it
				else                        --      else
				    exit                    --        jump out
				end if
			    end for
			    puts(1, '\n')
			    system("pause", 0)              --   let user read
			  end if                            --  continue
			end if
		    end while
		    close(fn)                           --  close the file
		end if
	    end if
	end for
    end for
end procedure

function accept_word_phrase(sequence index)
-- Accepts user's partial name for a function or procedure and offers better
-- suggestions; the one picked by the user is returned
    sequence scr_text, posn, search_word, pick_list
    object getter
    atom v, p
    clear_screen()
    -- Introductory text:
    scr_text = {
      "INCLUDE.EX: A utility for use with Euphoria\n\n",
      " Program searches directory \"c:\\euphoria\\include\" for GLOBAL\n",
      " procedures and functions, based on the word/phrase you type.\n\n"
    }
    for i = 1 to length(scr_text) do            -- show introductory text
	puts(1, scr_text[i])
    end for
    posn = get_position()                       -- note cursor position
    getter = "\n"                               -- initial the loop
    while equal(getter, "\n") do                -- get user's word or phrase
	blankline(posn[1])
	puts(1, " Type a word or phrase from the name of the function: ")
	getter = gets(0)
	getter = ltrim(getter)
    end while
    search_word = getter[1..$-1]                -- chop newline
    pick_list = {}                              -- initial pick list
    for i = 1 to length(index) do               -- for each item in index
	for j = 1 to length(index[i][2]) do     --   traverse the files topics  
	    -- must pass over word 'function' or 'procedure'
	    p = find(' ', index[i][2][j])       --     seek space after word
	    if match(search_word, index[i][2][j][p..$]) then -- if match,
		pick_list = append(pick_list, index[i][2][j])--   append do list
	    end if
	end for
    end for
    if length(pick_list) > 1 then       -- if multiple entries, user picks.
	blankline(posn[1])
	puts(1, "The following matching functions/procedures were found:\n\n")
	for i = 1 to length(pick_list) do
	    printf(1, "  %3d. %s\n", {i, pick_list[i]})
	end for
	posn = get_position()
	v = 0
	while v < 1 or v > length(pick_list) do
	    blankline(posn[1])
	    puts(1, "\n Type the number of the one to examine: ")
	    getter = gets(0)
	    getter = value(getter)
	    if getter[1] = GET_SUCCESS then
		v = floor(getter[2])
	    end if
	end while
	search_word = pick_list[v]
    elsif length(pick_list) = 1 then    -- if only one entry, use it
	search_word = pick_list[1]
    else                                -- if no entries, word/phrase not found
	puts(1, "\n\n No matching function/procedure was found.\n\n")
	system("pause", 0)
	search_word = ""
    end if
    return search_word
end function

procedure main()
    sequence file_list, index, searcher
    wrap(0)                             -- We'll turn off word wrap
    file_list = dir(path & "*.e")       -- build the file list
    index = build_index(file_list)      -- use file list to construct index
    searcher = accept_word_phrase(index)-- get item to search for from user
    if not equal(searcher, "") then     -- if item is not void
	clear_screen()                  --   display the relevant program text
	search_display(searcher, index)
    end if
    wrap(1)                             -- restore wrap
    if graphics_mode(-1) then end if    -- purely protective
end procedure

main()   
